/* GLOBAL VARIABLE DECLARATIONS */
#include "defines.h";
#include <XBee.h>
#include <Servo.h>

// Array for receiving xBee comm
int arguments[MAX_NUMBER_OF_ARGUMENTS];

XBee xbee = XBee();  //Create a new XBee object, attached to the Arduino.

//
// XBeeAddress64 lampAdres = XBeeAddress64(0x0013A200, 0x40331975); // address of the Lamp_turret xBee, THIS IS ME!
XBeeAddress64 coAdres = XBeeAddress64(0x00000000, 0x00000000);
XBeeAddress64 sensAdres = XBeeAddress64(0x0013A200, 0x4033192E);    //address of the Sensor_turret


//
// flags and switches
boolean sendToCo=false;      // boolean for sending data to coordintor as well as to other turret
boolean servoTest=false;     // boolean for setting test servo modus, if false, then normal operation
boolean newSettings=true;

int state = STATE_STANDBY;   // see defines

//
// timing variables
long stepTime = 0;          // used for continuously slowly dimming the RGB lights to off
int stepInterval = 100;

int intervalA = 1000; // used for copyAngleandColor delay
long timeA = 0;

int intervalB = 1000; // used for sending startup command in STATE_SEND_DEMO_CMD
long timeB = 0;

// declaration for servo's
Servo servoZ;
Servo servoY;

int posZ = 90;      // position to write to servo
int maxZ = 180;    // maximum angle
int maxPosZ = 0;   // angle for maximum brightness detected
int minZ = 0;      // minimal angle
int dirZ = 1;      // direction
int prevDirZ = 0;  // previous direction, used for detecting when direction is turned
int stepZ = 1;     // stepsize

int posY = 90;      // position to write to servo
int minY = 0;      // minimal angle
int maxY = 180;    // maximum angle
int maxPosY = 0;   // angle for maximum brightness detected
int dirY = 1;      // direction
int prevDirY = 0;  // previous direction, used for detecting when direction is turned
int stepY = 180/20;// stepsize

// pin and variable declaration for RGB LED's
int redPin = 6;
int greenPin = 5;
int bluePin = 3;

int redVal = 0;    //  red brightness
int greenVal = 50;    //  green brightness
int blueVal = 0;     //  blue brightness

// pin and variable declaration for light sensor
int LDRpin = 1;
int LDRVal = 0;
int prevLDRVal = 0;



///////////
/* SETUP */
///////////
void setup () 
{
  xbee.begin(115200);

  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT);
  pinMode(sensorPin, INPUT);
  pinMode(ledPin, OUTPUT);

  servoY.attach(10);
  servoZ.attach(11);

  if (servoTest==true){
    state=100;
  }

  timeA=millis();
  timeB=millis();
}

///////////////
/* MAIN LOOP */
///////////////
void loop()
{
  // check for incoming XBee communication
  checkXBee();

  // slowly dim the last set light value
  if(redVal!=0 || greenVal!=0 || blueVal!=0){
    if (millis()>=stepTime+stepInterval){
      redVal--;
      redVal= max(redVal,0);
      greenVal--;
      greenVal=max(greenVal,0);
      blueVal--;
      blueVal=max(blueVal,0);

      newSettings=true;
      stepTime=millis();
    }
  }

  // handle changed brightnesses or angles
  if (newSettings==true){
    servoZ.write(posZ);
    servoY.write(posY);
    analogWrite(redPin, redVal);
    analogWrite(greenPin,greenVal);
    analogWrite(bluePin, blueVal);  
    newSettings=false; 
  }




  // state machine ///////////////
  switch (state){
    // 0
  case STATE_STANDBY:            // default, ready for anything
    break;

    // 10
  case STATE_COPY_DELAY:            // this only waits till intervalA has passed, then goes back to state 0
    if (millis()>=timeA+intervalA){
      newSettings=true; 
      state=STATE_STANDBY;
    }
    break;

    // 20
  case STATE_SEND_DEMO_CMD:
    if (millis()>=timeB+intervalB){
      redVal=50;
      newSettings=true;

      addArgument(1);                          // ask the sensor_turret to do 1 scan
      sendData(sensAdres,FUNCTION_SET_DEMO);

      // if send to coordinator is enabled
      if (sendToCo){
        addArgument(0);                          
        sendData(coAdres,FUNCTION_SWITCH_ON_OFF);
      }

      state= STATE_STANDBY;
    }
    break;

  case STATE_SEND_ON_OFF:
    if (millis()>=timeB+intervalB){
      redVal=50;
      newSettings=true;

      addArgument(0);                          // ask the sensor_turret to turn off
      sendData(sensAdres,FUNCTION_SWITCH_ON_OFF);

      // if send to coordinator is enabled
      if (sendToCo){
        addArgument(0);                          
        sendData(coAdres,FUNCTION_SWITCH_ON_OFF);
      }

      state= STATE_STANDBY;
    }
    break;

    // 100
  case STATE_SERVO_TEST:                        // for testing servo angles (blocking while executed!)
    if (servoTest==true){
      testServos();
      servoTest=false; // signal that servo test has been done 
      state=0; 
    }
    break;
  }

}







