// create reusable response objects for responses we expect to handle 
ZBRxResponse rx         = ZBRxResponse();
ModemStatusResponse msr = ModemStatusResponse();

void checkXBee()
{
  /** Algorithm
  1. Keep checking for new packets.
  2. If a new packet is received, process the data
  
  You can do other stuff in this loop
  **/
  xbee.readPacket();  //Reads all available serial bytes until a packet is parsed, an error occurs, or the buffer is empty. 
  
  if (xbee.getResponse().isAvailable()) //Returns a reference to the current response Note: once readPacket is called again this response will be overwritten!
  {
    readData(); 
  }
}


void readData() 
{
  /**
  ZBRxResponse means a message is received
  **/
  if (xbee.getResponse().isAvailable()) //Returns true if the response has been successfully parsed 
  {                                     //and is complete and ready for use. 
    if (xbee.getResponse().getApiId() == ZB_RX_RESPONSE)  
    { //Call with instance of ZBRxResponse class only if getApiId() == ZB_RX_RESPONSE to populate response.
      xbee.getResponse().getZBRxResponse(rx); 
      
      if (rx.getOption() == ZB_PACKET_ACKNOWLEDGED)
      {
        //the sender got an ACK
      } 
      else 
      {
        //we got it (obviously) but sender didn't get an ACK
      }
      
      //Store the ID of the function...
      int functionID  =  rx.getData(0);
      //...and count the number of arguments
      int numArgs     =  (rx.getDataLength()-1)/2;
      
      for (int i = 0; i < numArgs; i++) 
      { 
         int pos      =  (i*2) + 1;
         arguments[i] =  (rx.getData(pos) << 8) + rx.getData(pos+1);
      }
      
      functionTable(functionID, numArgs);
    }
    
    /**
    The local XBee sends this response on certain events, like association/dissociation
    **/
    else if (xbee.getResponse().getApiId() == MODEM_STATUS_RESPONSE) 
    {
      xbee.getResponse().getModemStatusResponse(msr);
      
      if (msr.getStatus() == ASSOCIATED) 
      { //When associated a new network
      } 
      else if (msr.getStatus() == DISASSOCIATED)
      { //When disassociated with a network
      } 
      else 
      {
      }
    }
    
    /**
    Something occured that is unknown, or we do not care about
    **/
    else 
    { 
    }
    
  } 
}
